@@ -3,9 +3,10 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2020, University of Amsterdam
+    Copyright (c)  1985-2023, University of Amsterdam
                               VU University Amsterdam
                               CWI, Amsterdam
+                              SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -70,7 +71,10 @@
 user:file_search_path(autoload, swi(library)).
 user:file_search_path(autoload, pce(prolog/lib)).
 user:file_search_path(autoload, app_config(lib)).
+user:file_search_path(autoload, Dir) :-
+    '$ext_library_directory'(Dir).
 
+:- create_prolog_flag(warn_autoload, false, []).
 
 %!  '$find_library'(+Module, +Name, +Arity, -LoadModule, -Library) is semidet.
 %
@@ -488,7 +492,7 @@ exports_(File, Module, Exports) :-
             nonvar(Lib),
             arg(2, State, M),
             atom(M)
-        ->  use_foreign_library(M:Lib),
+        ->  catch('$syspreds':use_foreign_library_noi(M:Lib), error(_,_), true),
             fail
         ;   Term = (:- Directive),
             nonvar(Directive)
@@ -637,7 +641,8 @@ do_autoload(Library, Module:Name/Arity, LoadModule) :-
             \+ '$loading'(Library)
         ->  Module:import(LoadModule:Name/Arity)
         ;   use_module(Module:Library, [Name/Arity])
-        )
+        ),
+        warn_autoload(Module, LoadModule:Name/Arity)
     ),
     '$set_compilation_mode'(OldComp),
     '$set_autoload_level'(Old),
@@ -656,7 +661,7 @@ verbose_autoload(PI, Library) :-
 %!  autoloadable(:Head, -File) is nondet.
 %
 %   True when Head can be  autoloaded   from  File.  This implements the
-%   predicate_property/2 property autoload(File).  The   module  muse be
+%   predicate_property/2 property autoload(File).  The   module  must be
 %   instantiated.
 
 :- public                               % used from predicate_property/2
@@ -734,13 +739,15 @@ library_info(Spec, Context, FullFile, Module, Exports) :-
         fail
     ).
 
-
 library_info_from_file(FullFile, Module, Exports) :-
     setup_call_cleanup(
-        '$open_source'(FullFile, In, State, [], []),
-        '$term_in_file'(In, _Read, _RLayout, Term, _TLayout, _Stream,
-                        [FullFile], []),
-        '$close_source'(State, true)),
+        '$set_source_module'(OldModule, system),
+        setup_call_cleanup(
+            '$open_source'(FullFile, In, State, [], []),
+            '$term_in_file'(In, _Read, _RLayout, Term, _TLayout, _Stream,
+                            [FullFile], []),
+            '$close_source'(State, true)),
+        '$set_source_module'(OldModule)),
     (   Term = (:- module(Module, Exports))
     ->  !
     ;   nonvar(Term),
@@ -923,6 +930,32 @@ current_autoload(M:File, Context, Term) :-
     '$get_predicate_attribute'(M:'$autoload'(_,_,_), defined, 1),
     M:'$autoload'(File, Context, Term).
 
+		 /*******************************
+		 *            CHECK		*
+		 *******************************/
+
+warn_autoload(TargetModule, PI) :-
+    current_prolog_flag(warn_autoload, true),
+    \+ current_prolog_flag(xref, true),
+    \+ nb_current('$autoload_warning', true),
+    '$pi_head'(PI, Head),
+    source_file(Head, File),
+    expansion_hook(P),
+    source_file(P, File),
+    !,
+    setup_call_cleanup(
+        b_setval('$autoload_warning', true),
+        print_message(warning,
+                      deprecated(autoload(TargetModule, File, PI, expansion))),
+        nb_delete('$autoload_warning')).
+warn_autoload(_, _).
+
+expansion_hook(user:goal_expansion(_,_)).
+expansion_hook(user:goal_expansion(_,_,_,_)).
+expansion_hook(system:goal_expansion(_,_)).
+expansion_hook(system:goal_expansion(_,_,_,_)).
+
+
                  /*******************************
                  *             REQUIRE          *
                  *******************************/
