@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2021, University of Amsterdam
+    Copyright (c)  1985-2022, University of Amsterdam
                               VU University Amsterdam
                               CWI, Amsterdam
                               SWI-Prolog Solutions b.v.
@@ -48,8 +48,6 @@
             unload_file/1,
             exists_source/1,                    % +Spec
             exists_source/2,                    % +Spec, -Path
-            use_foreign_library/1,		% :FileSpec
-            use_foreign_library/2,		% :FileSpec, +Install
             prolog_load_context/2,
             stream_position_data/3,
             current_predicate/2,
@@ -67,8 +65,6 @@
             shell/1,                            % +Command
             on_signal/3,
             current_signal/3,
-            open_shared_object/2,
-            open_shared_object/3,
             format/1,
             garbage_collect/0,
             set_prolog_stack/2,
@@ -98,8 +94,6 @@
 
 :- meta_predicate
     dynamic(:, +),
-    use_foreign_library(:),
-    use_foreign_library(:, +),
     transaction(0),
     transaction(0,0,+),
     snapshot(0),
@@ -481,6 +475,8 @@ unload_file(File) :-
     ;   true
     ).
 
+:- if(current_prolog_flag(open_shared_object, true)).
+
 		 /*******************************
 		 *      FOREIGN LIBRARIES	*
 		 *******************************/
@@ -502,13 +498,24 @@ unload_file(File) :-
 %   _immediately_. I.e. the  difference  is   only  relevant  if the
 %   remainder of the file uses functionality of the C-library.
 
+:- meta_predicate
+    use_foreign_library(:),
+    use_foreign_library(:, +).
+:- public
+    use_foreign_library_noi/1.
+
 use_foreign_library(FileSpec) :-
     ensure_shlib,
-    initialization(shlib:load_foreign_library(FileSpec), now).
+    initialization(use_foreign_library_noi(FileSpec), now).
 
-use_foreign_library(FileSpec, Entry) :-
+% noi -> no initialize; used by '$autoload':exports/3.
+use_foreign_library_noi(FileSpec) :-
     ensure_shlib,
-    initialization(shlib:load_foreign_library(FileSpec, Entry), now).
+    shlib:load_foreign_library(FileSpec).
+
+use_foreign_library(FileSpec, Options) :-
+    ensure_shlib,
+    initialization(shlib:load_foreign_library(FileSpec, Options), now).
 
 ensure_shlib :-
     '$get_predicate_attribute'(shlib:load_foreign_library(_), defined, 1),
@@ -517,6 +524,50 @@ ensure_shlib :-
 ensure_shlib :-
     use_module(library(shlib), []).
 
+:- export(use_foreign_library/1).
+:- export(use_foreign_library/2).
+
+:- elif(current_predicate('$activate_static_extension'/1)).
+
+% Version when using shared objects is disabled and extensions are added
+% as static libraries.
+
+:- meta_predicate
+    use_foreign_library(:).
+:- public
+    use_foreign_library_noi/1.
+:- dynamic
+    loading/1,
+    foreign_predicate/2.
+
+use_foreign_library(FileSpec) :-
+    initialization(use_foreign_library_noi(FileSpec), now).
+
+use_foreign_library_noi(Module:foreign(Extension)) :-
+    setup_call_cleanup(
+        asserta(loading(foreign(Extension)), Ref),
+        @('$activate_static_extension'(Extension), Module),
+        erase(Ref)).
+
+:- export(use_foreign_library/1).
+
+system:'$foreign_registered'(M, H) :-
+    (   loading(Lib)
+    ->  true
+    ;   Lib = '<spontaneous>'
+    ),
+    assert(foreign_predicate(Lib, M:H)).
+
+%!  current_foreign_library(?File, -Public)
+%
+%   Query currently loaded shared libraries.
+
+current_foreign_library(File, Public) :-
+    setof(Pred, foreign_predicate(File, Pred), Public).
+
+:- export(current_foreign_library/2).
+
+:- endif. /* open_shared_object support */
 
                  /*******************************
                  *            STREAMS           *
@@ -832,7 +883,7 @@ define_or_generate(Pred) :-
     '$get_predicate_attribute'(Pred, size, Bytes).
 
 system_undefined(user:prolog_trace_interception/4).
-system_undefined(user:prolog_exception_hook/4).
+system_undefined(prolog:prolog_exception_hook/5).
 system_undefined(system:'$c_call_prolog'/0).
 system_undefined(system:window_title/2).
 
@@ -869,7 +920,7 @@ visible_predicate(Pred) :-
     ->  (   '$get_predicate_attribute'(Pred, defined, 1)
         ->  true
         ;   \+ current_prolog_flag(M:unknown, fail),
-            functor(Head, Name, Arity),
+            '$head_name_arity'(Head, Name, Arity),
             '$find_library'(M, Name, Arity, _LoadModule, _Library)
         )
     ;   setof(PI, visible_in_module(M, PI), PIs),
@@ -1195,46 +1246,6 @@ prolog:called_by(on_signal(_,_,New), [New+1]) :-
     ), !, fail.
 
 
-                 /*******************************
-                 *            DLOPEN            *
-                 *******************************/
-
-%!  open_shared_object(+File, -Handle) is det.
-%!  open_shared_object(+File, -Handle, +Flags) is det.
-%
-%   Open a shared object or DLL file. Flags  is a list of flags. The
-%   following flags are recognised. Note   however  that these flags
-%   may have no affect on the target platform.
-%
-%       * =now=
-%       Resolve all symbols in the file now instead of lazily.
-%       * =global=
-%       Make new symbols globally known.
-
-open_shared_object(File, Handle) :-
-    open_shared_object(File, Handle, []). % use pl-load.c defaults
-
-open_shared_object(File, Handle, Flags) :-
-    (   is_list(Flags)
-    ->  true
-    ;   throw(error(type_error(list, Flags), _))
-    ),
-    map_dlflags(Flags, Mask),
-    '$open_shared_object'(File, Handle, Mask).
-
-dlopen_flag(now,        2'01).          % see pl-load.c for these constants
-dlopen_flag(global,     2'10).          % Solaris only
-
-map_dlflags([], 0).
-map_dlflags([F|T], M) :-
-    map_dlflags(T, M0),
-    (   dlopen_flag(F, I)
-    ->  true
-    ;   throw(error(domain_error(dlopen_flag, F), _))
-    ),
-    M is M0 \/ I.
-
-
                  /*******************************
                  *             I/O              *
                  *******************************/
@@ -1502,6 +1513,9 @@ set_prolog_gc_thread(Status) :-
     var(Status),
     !,
     '$instantiation_error'(Status).
+set_prolog_gc_thread(_) :-
+    \+ current_prolog_flag(threads, true),
+    !.
 set_prolog_gc_thread(false) :-
     !,
     set_prolog_flag(gc_thread, false),
@@ -1564,7 +1578,10 @@ undo(Goal) :-
 
 '$run_undo'([One]) :-
     !,
-    call(One).
+    (   call(One)
+    ->  true
+    ;   true
+    ).
 '$run_undo'(List) :-
     run_undo(List, _, Error),
     (   var(Error)
