@@ -128,6 +128,7 @@ load_init_file(Base, _) :-
                        [ access(read),
                          file_errors(fail)
                        ]),
+    !,
     asserta(loaded_init_file(Base, InitFile)),
     load_files(user:InitFile,
                [ scope_settings(false)
@@ -943,6 +944,20 @@ setup_interactive :-
                 *    USER INTERACTIVE LOOP      *
                 *********************************/
 
+%!  prolog:repl_loop_hook(+BeginEnd, +BreakLevel) is nondet.
+%
+%   Multifile  hook  that  allows  acting    on   starting/stopping  the
+%   interactive REPL loop. Called as
+%
+%       forall(prolog:repl_loop_hook(BeginEnd, BreakLevel), true)
+%
+%   @arg BeginEnd is one of `begin` or `end`
+%   @arg BreakLevel is 0 for the normal toplevel, -1 when
+%   non-interactive and >0 for _break environments_.
+
+:- multifile
+    prolog:repl_loop_hook/2.
+
 %!  prolog
 %
 %   Run the Prolog toplevel. This is now  the same as break/0, which
@@ -962,21 +977,30 @@ prolog :-
 %   exceptions are really unhandled (in Prolog).
 
 '$query_loop' :-
+    break_level(BreakLev),
+    setup_call_cleanup(
+        notrace(call_repl_loop_hook(begin, BreakLev)),
+        '$query_loop'(BreakLev),
+        notrace(call_repl_loop_hook(end, BreakLev))).
+
+call_repl_loop_hook(BeginEnd, BreakLev) :-
+    forall(prolog:repl_loop_hook(BeginEnd, BreakLev), true).
+
+
+'$query_loop'(BreakLev) :-
     current_prolog_flag(toplevel_mode, recursive),
     !,
-    break_level(Level),
-    read_expanded_query(Level, Query, Bindings),
+    read_expanded_query(BreakLev, Query, Bindings),
     (   Query == end_of_file
     ->  print_message(query, query(eof))
     ;   '$call_no_catch'('$execute_query'(Query, Bindings, _)),
         (   current_prolog_flag(toplevel_mode, recursive)
-        ->  '$query_loop'
+        ->  '$query_loop'(BreakLev)
         ;   '$switch_toplevel_mode'(backtracking),
-            '$query_loop'           % Maybe throw('$switch_toplevel_mode')?
+            '$query_loop'(BreakLev)     % Maybe throw('$switch_toplevel_mode')?
         )
     ).
-'$query_loop' :-
-    break_level(BreakLev),
+'$query_loop'(BreakLev) :-
     repeat,
         read_expanded_query(BreakLev, Query, Bindings),
         (   Query == end_of_file
@@ -985,7 +1009,7 @@ prolog :-
             (   current_prolog_flag(toplevel_mode, recursive)
             ->  !,
                 '$switch_toplevel_mode'(recursive),
-                '$query_loop'
+                '$query_loop'(BreakLev)
             ;   fail
             )
         ).
@@ -1317,7 +1341,7 @@ write_bindings(Bindings, ResidueVars, Delays, DetOrChp) :-
     '$current_typein_module'(TypeIn),
     translate_bindings(Bindings, Bindings1, ResidueVars, TypeIn:Residuals),
     omit_qualifier(Delays, TypeIn, Delays1),
-    name_vars(Bindings1, Residuals, Delays1),
+    name_vars(Bindings1, t(Residuals, Delays1)),
     write_bindings2(Bindings1, Residuals, Delays1, DetOrChp).
 
 write_bindings2([], Residuals, Delays, _) :-
@@ -1340,14 +1364,28 @@ write_bindings2(Bindings, Residuals, Delays, Chp) :-
         print_message(query, query(done))
     ).
 
-name_vars(Bindings, Residuals, Delays) :-
+%!  name_vars(+Bindings, +Term) is det.
+%
+%   Give a name ``_[A-Z][0-9]*`` to all variables   in Term, that do not
+%   have a name due to Bindings. Singleton   variables in Term are named
+%   `_`. The behavior depends on these Prolog flags:
+%
+%     - toplevel_name_variables
+%       Only act when `true`, else name_vars/2 is a no-op.
+%     - toplevel_print_anon
+%
+%   Variables are named by unifying them to `'$VAR'(Name)`
+%
+%   @arg Bindings is a list Name=Value
+
+name_vars(Bindings, Term) :-
     current_prolog_flag(toplevel_name_variables, true),
     !,
-    '$term_multitons'(t(Bindings,Residuals,Delays), Vars),
+    '$term_multitons'(t(Bindings,Term), Vars),
     name_vars_(Vars, Bindings, 0),
-    term_variables(t(Bindings,Residuals,Delays), SVars),
+    term_variables(t(Bindings,Term), SVars),
     anon_vars(SVars).
-name_vars(_Bindings, _Residuals, _Delays).
+name_vars(_Bindings, _Term).
 
 name_vars_([], _, _).
 name_vars_([H|T], Bindings, N) :-
@@ -1450,7 +1488,11 @@ collect_residual_goals([H|T]) -->
     prolog:translate_bindings(+, -, +, +, :).
 
 prolog:translate_bindings(Bindings0, Bindings, ResVars, ResGoals, Residuals) :-
-    translate_bindings(Bindings0, Bindings, ResVars, ResGoals, Residuals).
+    translate_bindings(Bindings0, Bindings, ResVars, ResGoals, Residuals),
+    name_vars(Bindings, t(ResVars, ResGoals, Residuals)).
+
+% should not be required.
+prolog:name_vars(Bindings, Term) :- name_vars(Bindings, Term).
 
 translate_bindings(Bindings0, Bindings, ResidueVars, Residuals) :-
     prolog:residual_goals(ResidueGoals, []),
