@@ -35,7 +35,7 @@ The second argument is ignored."
            (progn . ,body)
          (set-buffer-modified-p nil)
          (kill-buffer)
-         (sweeprolog-restart)
+         ;; (sweeprolog-restart)
          (setq-default sweeprolog-enable-flymake enable-flymake-flag)))))
 
 (defconst sweeprolog-tests-greeting
@@ -113,7 +113,7 @@ foo(Foo, Bar) :- flatten(Bar, Baz), member(Foo, Baz).
                         (help-at-pt-kbd-string))))
 
 (sweeprolog-deftest terms-at-point ()
-  "Test `sweeprolog-term-search'."
+  "Test `sweeprolog-terms-at-point'."
   "
 recursive(Var) :-
     (   true
@@ -148,16 +148,21 @@ bar(bar(bar), bar{bar:bar}, [bar,bar|bar]).
 foo([Bar|Baz]).
 "
   (goto-char (point-min))
-  (sweeprolog-term-search "bar")
-  (should (= (point) 10))
-  (sweeprolog-term-search "bar")
-  (should (= (point) 24))
-  (sweeprolog-term-search "bar")
-  (should (= (point) 31))
-  (sweeprolog-term-search "bar")
-  (should (= (point) 35))
-  (sweeprolog-term-search "bar")
-  (should (= (point) 39)))
+  (let ((unread-command-events (listify-key-sequence (kbd "RET"))))
+    (sweeprolog-term-search "bar"))
+  (should (= (point) 13))
+  (let ((unread-command-events (listify-key-sequence (kbd "RET"))))
+    (sweeprolog-term-search "bar"))
+  (should (= (point) 27))
+  (let ((unread-command-events (listify-key-sequence (kbd "RET"))))
+    (sweeprolog-term-search "bar"))
+  (should (= (point) 34))
+  (let ((unread-command-events (listify-key-sequence (kbd "RET"))))
+    (sweeprolog-term-search "bar"))
+  (should (= (point) 38))
+  (let ((unread-command-events (listify-key-sequence (kbd "RET"))))
+    (sweeprolog-term-search "bar"))
+  (should (= (point) 42)))
 
 (sweeprolog-deftest beginning-of-next-top-term-header ()
   "Test finding the beginning of the first top term."
@@ -459,11 +464,101 @@ baz(Baz) :- bar(Baz).
 "
                    )))
 
-(sweeprolog-deftest cap-variable ()
-  "Completion at point for variable names."
-  "baz(Baz) :- bar(B-!-)."
+(sweeprolog-deftest cap-option-functor ()
+  "Completion at point for predicate option functors."
+  "
+foo(T) :-
+    read_term(T, [va-!-mes(
+"
   (should (pcase (sweeprolog-completion-at-point)
-            (`(17 18 ("Baz") . ,_) t))))
+            (`(30 35 ("variable_names") . ,_) t))))
+
+(sweeprolog-deftest cap-option ()
+  "Completion at point for predicate options."
+  "
+foo(T) :-
+    read_term(T, [va-!-mes
+"
+  (let ((cap (sweeprolog-completion-at-point)))
+    (should (= 30 (nth 0 cap)))
+    (should (= 35 (nth 1 cap)))
+    (should (equal '("variable_names(_)")
+                   (all-completions "" (nth 2 cap))))))
+
+(sweeprolog-deftest cap-option-argument ()
+  "Completion at point for predicate option arguments."
+  "
+foo(T) :-
+    read_term(T, [syntax_errors(fa-!-
+"
+  (let ((cap (sweeprolog-completion-at-point)))
+    (should (= 44 (nth 0 cap)))
+    (should (= 46 (nth 1 cap)))
+    (should (equal '("fail") (nth 2 cap)))))
+
+(sweeprolog-deftest cap-arith-functor ()
+  "Completion at point for arithmetic function functors."
+  "
+foo(T) :-
+    T is cop-!-n(
+"
+  (let ((cap (sweeprolog-completion-at-point)))
+    (should (= 21 (nth 0 cap)))
+    (should (= 25 (nth 1 cap)))
+    (should (equal '("copysign")
+                   (all-completions "" (nth 2 cap))))))
+
+(sweeprolog-deftest cap-arith ()
+                    "Completion at point for arithmetic functions."
+                    "
+foo(T) :-
+    T is cop-!-
+"
+                    (let ((cap (sweeprolog-completion-at-point)))
+                      (should (= 21 (nth 0 cap)))
+                      (should (= 24 (nth 1 cap)))
+                      (should (equal '("copysign(_, _)")
+                                     (all-completions "" (nth 2 cap))))))
+
+(sweeprolog-deftest cap-arith-nested ()
+  "Completion at point for arithmetic functions."
+  "
+foo(T) :-
+    T is copysign(cop-!-
+"
+  (let ((cap (sweeprolog-completion-at-point)))
+    (should (= 30 (nth 0 cap)))
+    (should (= 33 (nth 1 cap)))
+    (should (equal '("copysign(_, _)")
+                   (all-completions "" (nth 2 cap))))))
+
+(sweeprolog-deftest cap-source ()
+  "Completion at point for source files."
+  ":- use_module(li-!-"
+  (should (pcase (sweeprolog-completion-at-point)
+            (`(15 17 ,candidates . ,_)
+             (member "library(prolog_colour)" candidates)))))
+
+(sweeprolog-deftest cap-source-in-library ()
+  "Completion at point for source files in a path alias."
+  ":- use_module(library(pro-!-"
+  (should (pcase (sweeprolog-completion-at-point)
+            (`(23 26 ,candidates . ,_)
+             (member "prolog_colour" candidates)))))
+
+(sweeprolog-deftest cap-source-alias-functor ()
+  "Completion at point for path alias functors."
+  ":- use_module(l-!-ry(prolog_source))."
+  (let ((res (sweeprolog-completion-at-point)))
+    (should (= (nth 0 res) 15))
+    (should (= (nth 1 res) 18))
+    (should (equal (nth 2 res) '("library")))))
+
+(sweeprolog-deftest cap-variable ()
+                    "Completion at point for variable names."
+                    "baz(Baz) :- bar(B-!-)."
+                    (should (pcase (sweeprolog-completion-at-point)
+                              (`(17 18 ("Baz") . ,_) t))))
 
 (sweeprolog-deftest cap-local-predicate ()
   "Completion at point for local predicates."
@@ -474,8 +569,8 @@ baz(Baz) :- fooba-!-"
   (let ((cap (sweeprolog-completion-at-point)))
     (should (= 64 (nth 0 cap)))
     (should (= 69 (nth 1 cap)))
-    (should (equal '(("foobar(Baz)" compound "term_position" 0 11 0 6 ((compound "-" 7 10))))
-                   (nth 2 cap)))))
+    (should (equal '("foobar(Baz)")
+                   (all-completions "" (nth 2 cap))))))
 
 (sweeprolog-deftest cap-autoloaded-predicate ()
   "Completion at point for remote predicates."
@@ -486,8 +581,8 @@ baz(Baz) :- lists:memberc-!-"
   (let ((cap (sweeprolog-completion-at-point)))
     (should (= 70 (nth 0 cap)))
     (should (= 77 (nth 1 cap)))
-    (should (equal '(("memberchk(Elem, List)" compound "term_position" 0 21 0 9 ((compound "-" 10 14) (compound "-" 16 20))))
-                   (nth 2 cap)))))
+    (should (equal '("memberchk(Elem, List)")
+                   (all-completions "" (nth 2 cap))))))
 
 (sweeprolog-deftest cap-compound ()
   "Completion at point for compound terms."
@@ -1765,4 +1860,469 @@ f:o(Bar) --> baz(Bar)."
   (forward-line)
   (should (string= (add-log-current-defun) "f:o//1")))
 
+(sweeprolog-deftest up-list ()
+  "Test `up-list' support."
+  "
+foo((A,B)) =>
+    (   bar(-!-A)
+    ;   baz(B)
+    ).
+"
+  (call-interactively #'up-list)
+  (should (= (point) 30))
+  (call-interactively #'up-list)
+  (should (= (point) 51)))
+
+(sweeprolog-deftest extract-region-to-predicate ()
+  "Test `sweeprolog-extract-region-to-predicate'."
+  "
+:- module(bbb, []).
+
+bar(A, B, C, D, bar(bar), bar{bar:bar}, [bar,bar|bar]) :-
+    A = B,
+    C = D.
+"
+  (sweeprolog-extract-region-to-predicate 85 101 "bbb")
+  (should (string= (buffer-string)
+                   "
+:- module(bbb, []).
+
+bar(A, B, C, D, bar(bar), bar{bar:bar}, [bar,bar|bar]) :-
+    bbb(A, B, C, D).
+
+bbb(A, B, C, D) :-
+    A = B,
+    C = D.
+")))
+
+(sweeprolog-deftest extract-region-to-predicate-parens ()
+  "Test `sweeprolog-extract-region-to-predicate' with parentheses."
+  ""
+  (should (equal (sweeprolog--extract-goal "bar :-
+    (   A = B,
+        C = D
+    )."
+                                           11 41 "foo")
+                 (list "foo" "foo" ":-" "(   A = B,
+        C = D
+    )"
+                       "true" "foo" 0 nil)))
+  (should (equal (sweeprolog--extract-goal "bar :-
+    (   A = B,
+        C = D
+    )."
+                                           15 35 "foo")
+                 (list "foo" "foo" ":-" "A = B,
+        C = D"
+                       "true" "foo" 0 nil))))
+
+(sweeprolog-deftest extract-region-to-predicate-cut ()
+  "Test `sweeprolog-extract-region-to-predicate' in presence of a cut."
+  ""
+  (should (equal (sweeprolog--extract-goal "bar :-
+    A = B,
+    !,
+    C = D."
+                                           11 34 "foo")
+                 (list "foo" "foo" ":-" "A = B,
+    !,
+    C = D"
+                       nil "foo" 0 nil))))
+
+(sweeprolog-deftest extract-region-to-predicate-clean-cut ()
+  "Test `sweeprolog-extract-region-to-predicate' in presence of a clean cut."
+  ""
+  (should (equal (sweeprolog--extract-goal "bar :-
+    A = B,
+    call(!),
+    C = D."
+                                           11 40 "foo")
+                 (list "foo" "foo" ":-" "A = B,
+    call(!),
+    C = D"
+                       "true" "foo" 0 nil))))
+
+(sweeprolog-deftest extract-region-to-predicate-cut-1 ()
+  "Test `sweeprolog-extract-region-to-predicate' in presence of a cut."
+  ""
+  (should
+   (equal
+    (sweeprolog--extract-goal "
+bar :-
+    call((foo, ! ; baz)).
+"
+     12 32 "spam")
+    '("spam" "spam" ":-" "call((foo, ! ; baz))" "true" "spam" 0 nil))))
+
+(sweeprolog-deftest extract-region-to-predicate-cut-2 ()
+  "Test `sweeprolog-extract-region-to-predicate' in presence of a cut."
+  ""
+  (should
+   (equal
+    (sweeprolog--extract-goal "
+bar :-
+    call((foo, ! ; baz)).
+"
+     17 31 "spam")
+    '("spam" "spam" ":-" "(foo, ! ; baz)" "true" "spam" 0 nil))))
+
+(sweeprolog-deftest extract-region-to-predicate-cut-3 ()
+  "Test `sweeprolog-extract-region-to-predicate' in presence of a cut."
+  ""
+  (should
+   (equal
+    (sweeprolog--extract-goal "
+bar :-
+    call((foo, ! ; baz)).
+"
+     18 24 "spam")
+    '("spam" "spam" ":-" "foo, !" nil "spam" 0 nil))))
+
+(sweeprolog-deftest extract-region-to-predicate-dcg ()
+                    "Test `sweeprolog-extract-region-to-predicate'."
+                    ""
+                    (should (equal (sweeprolog--extract-goal "bar(A,D) -->
+    foo1(A, B),
+    foo2(C, D)."
+                                                             17 43 "foo")
+                                   (list "foo(A, D)" "foo(A, D)" "-->" "foo1(A, B),
+    foo2(C, D)"
+                                         "true" "foo" 2 nil))))
+
+(sweeprolog-deftest extract-region-to-predicate-dcg-to-reg-1 ()
+  "Test `sweeprolog-extract-region-to-predicate' with \"{}/1\" in DCG."
+  ""
+  (should (equal (sweeprolog--extract-goal "bar(A,D) -->
+    {foo1(A, B), foo2(C, D)}."
+                                           17 41 "foo")
+                 (list "foo(A, D)" "foo(A, D)" "-->" "{foo1(A, B), foo2(C, D)}"
+                       "true" "foo" 2 nil))))
+
+(sweeprolog-deftest extract-region-to-predicate-dcg-to-reg-2 ()
+  "Test `sweeprolog-extract-region-to-predicate' with \"{}/1\" in DCG."
+  ""
+  (should (equal (sweeprolog--extract-goal "bar(A,D) -->
+    {foo1(A, B), foo2(C, D)}."
+                                           18 40 "foo")
+                 (list "foo(A, D)" "foo(A, D)" ":-" "foo1(A, B), foo2(C, D)"
+                       "true" "foo" 2 nil))))
+
+(sweeprolog-deftest extract-region-to-predicate-dcg-in-use ()
+  "Test `sweeprolog-extract-region-to-predicate' with DCG that's in use."
+  ":- module(baz, []).
+
+bar(A,D) -->
+    foo1(A, B),
+    foo2(C, D).
+
+foo(_,_) --> [].
+"
+  (should (equal (sweeprolog--extract-goal "bar(A,D) -->
+    foo1(A, B),
+    foo2(C, D)."
+                                           17 43 "foo")
+                 (list "foo(A, D)" "foo(A, D)" "-->" "foo1(A, B),
+    foo2(C, D)"
+                       "true" "foo" 2 "true"))))
+
+
+(sweeprolog-deftest extract-region-to-predicate-1 ()
+  "Test `sweeprolog-extract-region-to-predicate'."
+  "
+:- module(bbb, []).
+
+bar(A, B, C, D, bar(bar), bar{bar:bar}, [bar,bar|bar]) :-
+    A = B,
+    C = D.
+"
+  (sweeprolog-extract-region-to-predicate 85 90 "bbb")
+  (should (string= (buffer-string)
+                   "
+:- module(bbb, []).
+
+bar(A, B, C, D, bar(bar), bar{bar:bar}, [bar,bar|bar]) :-
+    bbb(A, B),
+    C = D.
+
+bbb(A, B) :-
+    A = B.
+")))
+
+(sweeprolog-deftest extract-region-to-predicate-2 ()
+  "Test `sweeprolog-extract-region-to-predicate'."
+  "
+:- module(bbb, []).
+
+bar(A, B) :-
+    (   A = C,
+        B = D
+    ;   A = C,
+        B = D
+    ).
+"
+  (sweeprolog-extract-region-to-predicate 44 64 "bbb")
+  (should (string= (buffer-string)
+                   "
+:- module(bbb, []).
+
+bar(A, B) :-
+    (   bbb(A, B)
+    ;   A = C,
+        B = D
+    ).
+
+bbb(A, B) :-
+    A = C,
+    B = D.
+")))
+
+(sweeprolog-deftest extract-region-to-predicate-3 ()
+  "Test `sweeprolog-extract-region-to-predicate'."
+  "
+:- module(bbb, []).
+
+bar(A, B) :-
+    (   A = C,
+        B = D
+    ;   A = C,
+        B = D
+    ).
+"
+  (sweeprolog-extract-region-to-predicate 73 93 "bbb")
+  (should (string= (buffer-string)
+                   "
+:- module(bbb, []).
+
+bar(A, B) :-
+    (   A = C,
+        B = D
+    ;   bbb(A, B)
+    ).
+
+bbb(A, B) :-
+    A = C,
+    B = D.
+")))
+
+(sweeprolog-deftest extract-region-to-predicate-4 ()
+  "Test `sweeprolog-extract-region-to-predicate'."
+  "
+:- module(bbb, []).
+
+bar(A, B) :-
+    A = C,
+    B = D,
+    A = C,
+    B = D.
+"
+  (sweeprolog-extract-region-to-predicate 40 56 "bbb")
+  (should (string= (buffer-string)
+                   "
+:- module(bbb, []).
+
+bar(A, B) :-
+    bbb(A, B, C, D),
+    A = C,
+    B = D.
+
+bbb(A, B, C, D) :-
+    A = C,
+    B = D.
+")))
+
+(sweeprolog-deftest extract-region-to-predicate-5 ()
+  "Test `sweeprolog-extract-region-to-predicate'."
+  "
+:- module(bbb, []).
+
+bar(A, B) :-
+    A = C,
+    B = D,
+    A = C,
+    B = D.
+"
+  (sweeprolog-extract-region-to-predicate 51 67 "bbb")
+  (should (string= (buffer-string)
+                   "
+:- module(bbb, []).
+
+bar(A, B) :-
+    A = C,
+    bbb(A, B, C, D),
+    B = D.
+
+bbb(A, B, C, D) :-
+    B = D,
+    A = C.
+")))
+
+(sweeprolog-deftest extract-region-to-predicate-6 ()
+  "Test `sweeprolog-extract-region-to-predicate'."
+  "
+:- module(bbb, []).
+
+bar(A, B) :-
+    A = C,
+    B = D,
+    A = C,
+    B = D.
+"
+  (sweeprolog-extract-region-to-predicate 62 78 "bbb")
+  (should (string= (buffer-string)
+                   "
+:- module(bbb, []).
+
+bar(A, B) :-
+    A = C,
+    B = D,
+    bbb(A, B, C, D).
+
+bbb(A, B, C, D) :-
+    A = C,
+    B = D.
+")))
+
+(sweeprolog-deftest extract-region-to-predicate-7 ()
+  "Test `sweeprolog-extract-region-to-predicate'."
+  "
+:- module(bbb, []).
+
+bar(A, B) :-
+    (   A = B
+    ->  C = D
+    ;   (   A = B -> C = D   )
+    ;   A = E, E = B
+    ->  C = D
+    ).
+"
+  (sweeprolog-extract-region-to-predicate 72 94 "bbb")
+  (should (string= (buffer-string)
+                   "
+:- module(bbb, []).
+
+bar(A, B) :-
+    (   A = B
+    ->  C = D
+    ;   bbb(A, B)
+    ;   A = E, E = B
+    ->  C = D
+    ).
+
+bbb(A, B) :-
+    (   A = B -> C = D   ).
+")))
+
+(sweeprolog-deftest extract-region-to-predicate-8 ()
+  "Test `sweeprolog-extract-region-to-predicate'."
+  "
+:- module(bbb, []).
+
+bar(A, B) :-
+    (   A = B
+    ->  C = D
+    ;   (   A = B -> C = D   )
+    ;   A = E, E = B
+    ->  C = D
+    ).
+"
+  (sweeprolog-extract-region-to-predicate 103 115 "bbb")
+  (should (string= (buffer-string)
+                   "
+:- module(bbb, []).
+
+bar(A, B) :-
+    (   A = B
+    ->  C = D
+    ;   (   A = B -> C = D   )
+    ;   bbb(A, B)
+    ->  C = D
+    ).
+
+bbb(A, B) :-
+    A = E, E = B.
+")))
+
+(sweeprolog-deftest extract-region-to-predicate-ext-1 ()
+                    "Test `sweeprolog-extract-region-to-predicate'."
+                    "
+:- module(bbb, []).
+
+bar(A, Y) :-
+    setof(X, Y^member(X, Y), Y).
+"
+                    (sweeprolog-extract-region-to-predicate 49 63 "bbb")
+                    (should (string= (buffer-string)
+                                     "
+:- module(bbb, []).
+
+bar(A, Y) :-
+    setof(X, bbb(X), Y).
+
+bbb(X) :-
+    member(X, Y).
+")))
+
+(sweeprolog-deftest extract-region-to-predicate-ext-2 ()
+  "Test `sweeprolog-extract-region-to-predicate'."
+  "
+:- module(bbb, []).
+
+bar(A, Y) :-
+    setof(X, Y^(member(X, Y), X = Z), Y).
+"
+  (sweeprolog-extract-region-to-predicate 51 72 "bbb")
+  (should (string= (buffer-string)
+                   "
+:- module(bbb, []).
+
+bar(A, Y) :-
+    setof(X, Y^bbb(Y, X), Y).
+
+bbb(Y, X) :-
+    (member(X, Y), X = Z).
+")))
+
+(sweeprolog-deftest extract-region-to-predicate-lambda-1 ()
+  "Test `sweeprolog-extract-region-to-predicate'."
+  "
+:- module(bbb, []).
+
+bar(A, Y) :-
+    maplist([VarName]>>ignore(memberchk(VarName, GoalVarNames)),
+            TemplateVarNames).
+"
+  (sweeprolog-extract-region-to-predicate 48 99 "bbb")
+  (should (string= (buffer-string)
+                   "
+:- module(bbb, []).
+
+bar(A, Y) :-
+    maplist(bbb,
+            TemplateVarNames).
+
+bbb(VarName) :-
+    ignore(memberchk(VarName, GoalVarNames)).
+")))
+
+(sweeprolog-deftest extract-region-to-predicate-lambda-2 ()
+                    "Test `sweeprolog-extract-region-to-predicate'."
+                    "
+:- module(bbb, []).
+
+bar(A, Y) :-
+    maplist({GoalVarNames}/[VarName]>>ignore(memberchk(VarName, GoalVarNames)),
+            TemplateVarNames).
+"
+                    (sweeprolog-extract-region-to-predicate 48 114 "bbb")
+                    (should (string= (buffer-string)
+                                     "
+:- module(bbb, []).
+
+bar(A, Y) :-
+    maplist(bbb(GoalVarNames),
+            TemplateVarNames).
+
+bbb(GoalVarNames, VarName) :-
+    ignore(memberchk(VarName, GoalVarNames)).
+")))
 ;;; sweeprolog-tests.el ends here
