@@ -62,7 +62,9 @@ static atom_t ATOM_locals;
 static atom_t ATOM_globals;
 static atom_t ATOM_dict;
 
-static functor_t FUNCTOR_python_error3;
+static functor_t FUNCTOR_python_error2;
+static functor_t FUNCTOR_python_stack1;
+static functor_t FUNCTOR_context2;
 static functor_t FUNCTOR_error2;
 static functor_t FUNCTOR_module2;
 static functor_t FUNCTOR_eq2;
@@ -172,7 +174,7 @@ __sync_bool_compare_and_swap(volatile void **addr, void *old, void *new)
 #if _WIN64
   return _InterlockedCompareExchange64((volatile __int64*)addr, (__int64)new, (__int64)old) == (__int64)old;
 #else
-  return _InterlockedCompareExchange64((volatile long*)addr, (long)new, (long)old) == (long)old;
+  return _InterlockedCompareExchange((volatile long*)addr, (long)new, (long)old) == (long)old;
 #endif
 }
 
@@ -363,11 +365,11 @@ get_py_obj(term_t t, PyObject **objp, int error)
       *objp = obj;
       return TRUE;
     } else
-      return PL_existence_error("PyObject", t);
+      return PL_existence_error("py_object", t);
   }
 
   if ( error )
-    PL_type_error("PyObject", t);
+    PL_type_error("py_object", t);
 
   return FALSE;
 }
@@ -471,20 +473,26 @@ check_error(PyObject *obj)
 
   if ( ex )
   { PyObject *type = NULL, *tname = NULL, *value = NULL, *stack = NULL;
-    term_t t  = PL_new_term_ref();
-    term_t av = PL_new_term_refs(3);
+    term_t t   = PL_new_term_ref();
+    term_t av  = PL_new_term_refs(2);
+    term_t ctx = PL_new_term_ref();
 
     PyErr_Fetch(&type, &value, &stack);
+    if ( stack )
+    { if ( !py_unify(ctx, stack, PYU_ERROR) ||
+	   !PL_cons_functor_v(ctx, FUNCTOR_python_stack1, ctx) ||
+	   !PL_cons_functor(ctx, FUNCTOR_context2, t, ctx) )
+	return NULL;
+      PL_put_variable(t);
+    }
+
     if ( (tname=PyObject_GetAttrString(type, "__name__")) &&
          PL_unify_chars(av+0, PL_ATOM|REP_UTF8, (size_t)-1,
 			PyUnicode_AsUTF8AndSize(tname, NULL)) &&
 	 (value ? py_unify(av+1, value, PYU_ERROR)
 		: py_unify_constant(av+1, ATOM_none)) &&
-	 (stack ? py_unify(av+2, stack, PYU_ERROR)
-		: py_unify_constant(av+2, ATOM_none)) &&
-	 PL_cons_functor_v(t, FUNCTOR_python_error3, av) &&
-	 PL_put_variable(av+0) &&
-	 PL_cons_functor(t, FUNCTOR_error2, t, av+0) )
+	 PL_cons_functor_v(t, FUNCTOR_python_error2, av) &&
+	 PL_cons_functor(t, FUNCTOR_error2, t, ctx) )
       PL_raise_exception(t);
 
     Py_CLEAR(tname);
@@ -970,7 +978,7 @@ py_add_to_dict(term_t key, term_t value, void *closure)
 static int
 add_prolog_key_value_to_dict(PyObject *py_dict, term_t tuple,
 			     term_t key, term_t value)
-{ if ( PL_is_functor(tuple, FUNCTOR_key_value2) )
+{ if ( PL_is_functor(tuple, FUNCTOR_key_value2) ) /* Key:Value */
   { _PL_get_arg(1, tuple, key);
     _PL_get_arg(2, tuple, value);
     PyObject *py_key = NULL, *py_value = NULL;
@@ -1343,7 +1351,7 @@ py_from_prolog(term_t t, PyObject **obj)
     return TRUE;
 
 error:
-  return PL_domain_error("py_data", t),FALSE;
+  return PL_domain_error("py_term", t),FALSE;
 }
 
 
@@ -2272,12 +2280,12 @@ py_free(term_t t)
     (void)data;
     if ( type == &PY_OBJECT )
     { if ( size == 0 )
-	return PL_existence_error("PyObject", t);
+	return PL_existence_error("py_object", t);
       return PL_free_blob(blob);
     }
   }
 
-  return PL_type_error("py_obj", t);
+  return PL_type_error("py_object", t);
 }
 
 
@@ -2393,9 +2401,11 @@ install_janus(void)
   ATOM_pydict = PL_new_atom("py");
   ATOM_curl   = PL_new_atom("{}");
 
-  MKFUNCTOR(python_error, 3);
+  MKFUNCTOR(python_error, 2);
+  MKFUNCTOR(python_stack, 1);
   MKFUNCTOR(error, 2);
   MKFUNCTOR(py, 1);
+  MKFUNCTOR(context, 2);
   FUNCTOR_module2    = PL_new_functor(PL_new_atom(":"), 2);
   FUNCTOR_eq2        = PL_new_functor(PL_new_atom("="), 2);
   FUNCTOR_hash1      = PL_new_functor(PL_new_atom("#"), 1);
