@@ -87,6 +87,7 @@
 :- autoload(library(prolog_code), [comma_list/2]).
 :- autoload(library(readutil), [read_line_to_string/2]).
 :- autoload(library(wfs), [call_delays/2, delays_residual_program/2]).
+:- autoload(library(dcg/high_order), [sequence//2, sequence//3]).
 
 :- if(\+current_predicate(py_call/1)).
 :- if(current_prolog_flag(windows, true)).
@@ -123,6 +124,9 @@ add_python_dll_dir :-
     py_write/2,
     py_readline/4.
 
+:- create_prolog_flag(py_backtrace,       true, [type(boolean), keep(true)]).
+:- create_prolog_flag(py_backtrace_depth, 4,    [type(integer), keep(true)]).
+
 /** <module> Call Python from Prolog
 
 This library implements calling Python  from   Prolog.  It  is available
@@ -144,8 +148,9 @@ search path may be extended using py_add_lib_dir/1.
 %   active, indicate this with the location of this environment found.
 
 py_version :-
-    py_call(sys:version, X),
-    print_message(information, janus(version(X))),
+    py_call(sys:version, PythonVersion),
+    py_call(janus_swi:version_str(), JanusVersion),
+    print_message(information, janus(version(JanusVersion, PythonVersion))),
     (   py_venv(VEnvDir, EnvSiteDir)
     ->  print_message(information, janus(venv(VEnvDir, EnvSiteDir)))
     ;   true
@@ -408,7 +413,7 @@ is_kv(K:V) => ground(K), ground(V).
 %   details.
 %
 %   @compat  PIP.  See  py_call/2  for  notes.    Note   that,  as  this
-%   implementation is based on  py_call/2,   Function  can use changing,
+%   implementation is based on py_call/2,   Function can use _chaining_,
 %   e.g., py_func(sys, path:append(dir), Return)  is   accepted  by this
 %   implementation, but not portable.
 
@@ -663,8 +668,7 @@ py_obj_dict(ObjRef, Dict) :-
 %   Source is a no-op. Called with  a   different  source  creates a new
 %   Python module that replaces the old in the global namespace.
 %
-%   @error python_error(Type, Data, Stack) is raised if Python raises an
-%   error.
+%   @error python_error(Type, Data) is raised if Python raises an error.
 
 :- dynamic py_dyn_module/2 as volatile.
 
@@ -686,6 +690,7 @@ py_module(Module, Source) :-
 		 *******************************/
 
 :- dynamic py_venv/2 as volatile.
+:- dynamic py_is_initialized/0 as volatile.
 
 %   py_initialize is det.
 %
@@ -762,7 +767,9 @@ py_initialize(Program, Argv, Options) :-
 			   [ access(read) ]),
 	file_directory_name(Janus, PythonDir),
 	py_add_lib_dir(PythonDir, first),
-	py_connect_io
+	py_connect_io,
+        repl_add_cwd,
+        asserta(py_is_initialized)
     ;   true
     ).
 
@@ -804,20 +811,20 @@ py_lib_dirs(Dirs) :-
 %
 %   Add a directory to the Python  module   search  path.  In the second
 %   form, Where is one of `first`   or `last`. py_add_lib_dir/1 adds the
-%   directory as first. The property `sys:path`   is  not modified if it
+%   directory as `last`. The property `sys:path`   is not modified if it
 %   already contains Dir.
 %
 %   Dir is in Prolog notation. The added   directory  is converted to an
-%   absolute path using the OS notation.
+%   absolute path using the OS notation using prolog_to_os_filename/2.
 %
-%   The form py_add_lib_dir/0 may only be  used as a _directive_, adding
-%   the directory from which the current Prolog  source is loaded at the
-%   head  of  the   Python   search    path.   If   py_add_lib_dir/1  or
-%   py_add_lib_dir/2 are used in a directive  and the given directory is
-%   not absolute, it is  resolved  against   the  directory  holding the
-%   current Prolog source.
+%   If Dir is a _relative_ path, it   is taken relative to Prolog source
+%   file when used as a _directive_ and  relative to the process working
+%   directory when called as a predicate.
 %
-%   @compat PIP.  PIP only describes py_add_lib_dir/1.
+%   @compat PIP. Note  that  SWI-Prolog   uses  POSIX  file  conventions
+%   internally, mapping to OS  conventions   inside  the predicates that
+%   deal with files or explicitly   using prolog_to_os_filename/2. Other
+%   systems may use the native file conventions in Prolog.
 
 :- multifile system:term_expansion/2.
 
@@ -834,11 +841,14 @@ system:term_expansion((:- py_add_lib_dir(Dir0, Where)),
     absolute_file_name(Dir0, Dir).
 
 py_add_lib_dir(Dir) :-
-    py_add_lib_dir(Dir, first).
+    py_add_lib_dir(Dir, last).
 
 py_add_lib_dir(Dir, Where) :-
     absolute_file_name(Dir, AbsDir),
     prolog_to_os_filename(AbsDir, OSDir),
+    py_add_lib_dir_(OSDir, Where).
+
+py_add_lib_dir_(OSDir, Where) :-
     (   py_call(sys:path, Dirs0),
         memberchk(OSDir, Dirs0)
     ->  true
@@ -849,6 +859,24 @@ py_add_lib_dir(Dir, Where) :-
     ;   must_be(oneof([first,last]), Where)
     ).
 
+repl_add_cwd :-
+    current_prolog_flag(break_level, Level),
+    Level >= 0,
+    (   py_call(sys:path:count(''), N),
+        N > 0
+    ->  true
+    ;   print_message(informational, janus(add_cwd)),
+        py_add_lib_dir_('', first)
+    ).
+
+:- multifile
+    prolog:repl_loop_hook/2.
+
+prolog:repl_loop_hook(begin, Level) :-
+    Level >= 0,
+    py_is_initialized,
+    repl_add_cwd.
+
 
 		 /*******************************
 		 *           CALLBACK		*
@@ -1105,20 +1133,41 @@ py_consult(File, Data, Module) =>
 
 :- multifile
     prolog:error_message//1,
+    prolog:message_context//1,
     prolog:message//1.
 
-prolog:error_message(python_error(Class, Value, _Stack)) -->
+prolog:error_message(python_error(Class, Value)) -->
     { py_str(Value, Message)
     },
     [ 'Python ', ansi(code, "'~w'", [Class]), ':', nl,
       '  ~w'-[Message]
     ].
 
+prolog:message_context(context(_, PythonCtx)) -->
+    { nonvar(PythonCtx),
+      PythonCtx = python_stack(Stack),
+      current_prolog_flag(py_backtrace, true),
+      py_is_object(Stack),
+      !,
+      current_prolog_flag(py_backtrace_depth, Depth),
+      py_call(traceback:format_tb(Stack, Depth), Frames)
+    },
+    [ nl, 'Python stack:', nl ],
+    sequence(py_stack_frame, Frames).
+
+py_stack_frame(String) -->
+    { split_string(String, "\n", "", Lines)
+    },
+    sequence(msg_line, [nl], Lines).
+
+msg_line(Line) -->
+    [ '~s'-[Line] ].
+
 prolog:message(janus(Msg)) -->
     message(Msg).
 
-message(version(V)) -->
-    [ 'Janus embeds Python ~w'-[V] ].
+message(version(Janus, Python)) -->
+    [ 'Janus ~w embeds Python ~w'-[Janus, Python] ].
 message(venv(Dir, _EnvSiteDir)) -->
     [ 'Janus: using venv from ~p'-[Dir] ].
 message(venv(no_site_package_dir(VEnvDir, Dir))) -->
@@ -1127,3 +1176,5 @@ message(py_shell(no_janus)) -->
     [ 'Janus: py_shell/0: Importing janus into the Python shell requires Python 3.10 or later.', nl,
       'Run "', ansi(code, 'from janus import *', []), '" in the Python shell to import janus.'
     ].
+message(add_cwd) -->
+    [ 'Interactive session; added `.` to Python `sys.path`'-[] ].
